indexing

	description: "Class that covers POSIX memory mapping."

	to_do: "Perhaps enhance STDC_BUFFER that resizing is forbidden."

	author: "Berend de Boer"
	date: "$Date: 2003/10/02 $"
	revision: "$Revision: #3 $"

class

	POSIX_MEMORY_MAP

inherit

	ABSTRACT_MEMORY_MAP
		redefine
			close,
			dispose
		end

	POSIX_BUFFER
		undefine
			dispose
		end

	PAPI_MMAN


creation

	make,
	make_private,
	make_shared


feature -- Initialization

	make (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER;  a_base: POINTER; a_prot, a_flags: INTEGER) is
			-- Raw interface to mmap.
			-- This function can fail on certain system (Linux for
			-- example) if a_offset is not a multiple of PAGE_SIZE.
		require
			closed: not is_open
			have_file_descriptor: a_fd /= Void
			offset_not_negative: a_offset >= 0
			size_is_positive: a_size > 0
		do
			fd := a_fd
			offset := a_offset
			ptr := posix_mmap (a_base, a_size, a_prot, a_flags, fd.value, offset)
			if ptr = MAP_FAILED then
				raise_posix_error
			end
			make_from_pointer (ptr, a_size, False)
		ensure
			open: security.error_handling.exceptions_enabled implies is_open
		end

	make_private (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER ) is
			-- Make the given file descriptor. `a_fd' should have been opened
			-- with read/write access.
			-- This is a mapping where changes are private.
			-- `a_offset' denotes the offset from `a_fd'.
			-- This function can fail on certain system (Linux for
			-- example) if a_offset is not a multiple of PAGE_SIZE.
		require
			closed: not is_open
			have_file_descriptor: a_fd /= Void
			offset_not_negative: a_offset >= 0
			size_is_positive: a_size > 0
		do
			make (a_fd, a_offset, a_size, default_pointer, PROT_READ + PROT_WRITE, MAP_PRIVATE)
		ensure
			open: security.error_handling.exceptions_enabled implies is_open
		end

	make_shared (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER) is
			-- Make the given file descriptor. `a_fd' should have been opened
			-- with read/write access.
			-- This is a mapping where changes are shared, i.e. the
			-- `a_offset' denotes the offset from `a_fd'.
			-- underlying object is also changed.
			-- This function can fail on certain system (Linux for
			-- example) if a_offset is not a multiple of PAGE_SIZE.
		require
			closed: not is_open
			have_file_descriptor: a_fd /= Void
			offset_not_negative: a_offset >= 0
			size_is_positive: a_size > 0
		do
			make (a_fd, a_offset, a_size, default_pointer, PROT_READ + PROT_WRITE, MAP_SHARED)
		ensure
			open: security.error_handling.exceptions_enabled implies is_open
		end


feature -- Cleanup

	dispose is
		local
			r: INTEGER
		do
			if ptr /= default_pointer then
				r := posix_munmap (ptr, capacity)
			end
		rescue
			ptr := default_pointer
			retry
		end


feature -- Unmap

	close is
			-- Remove the mapping.
		do
			safe_call (posix_munmap (ptr, capacity))
			offset := 0
			precursor
		end


feature -- State

	offset: INTEGER
			-- Offset from file.

	fd: POSIX_FILE_DESCRIPTOR
			-- The file that is mapped.

invariant

	offset_not_negative: offset >= 0

end
