indexing

	description: "Converts strings to pointers and vice versa."
	thanks: "The mico/E team for the idea."

	date: "$Date: 2003/11/29 $"
	revision: "$Revision: #5 $"

class

	EPX_STRING_HELPER


inherit

	ABSTRACT_STRING_HELPER
		redefine
			do_uc_string_to_pointer
		end


feature -- SmartEiffel specific conversions

	pointer_to_string (p: POINTER): STRING is
			-- Returns a new string.
		do
			if p = default_pointer then
				create Result.make (0)
			else
				create Result.from_external_copy (p)
			end
		end

	set_string_from_pointer (s: STRING; p: POINTER) is
			-- copies contents of p to s
		do
			if p = default_pointer then
				s.clear
			else
				s.from_external_copy (p)
			end
		end

feature {NONE} -- Implementation

	clear_unsaved_strings is
			-- The strings in `saved_strings' must be released to the
			-- garbage collector upon `unfreeze_all'.
		do
			saved_strings.clear_all
			saved_strings.resize (0, -1)
		end

	do_string_to_pointer (s: STRING): POINTER is
			-- Return a pointer to a linear area containing all the data
			-- in `s'. The area is zero-terminated.
			-- 's'.count should be equal to `s'.byte_count.
		do
			Result := s.to_external
		end

	do_uc_string_to_pointer (s: UC_STRING): POINTER is
			-- Return a pointer to a linear area containing all the data
			-- in the Unicode string `s'.
			-- Currently getting such a pointer is not an operation
			-- directly supported by UC_STRING and we need to do
			-- different things for certain compilers.
		local
			real_s: STRING
		do
			-- SmartEiffel appends an '%U' to the string in to_external,
			-- and UC_STRING doesn't like that trick (and bombs). So
			-- always turn the UC_STRING into a real string.
			real_s := s.as_string
			Result := real_s.to_external
			-- A big issue now is to keep `real_s' alive until the next
			-- `unfreeze_all'.
			saved_strings.force (real_s, saved_strings.upper + 1)
		end


end
