indexing

	description: "Standard C last error number."

	author: "Berend de Boer"
	date: "$Date: 2003/07/10 $"
	revision: "$Revision: #4 $"

class

	STDC_ERRNO

inherit

	ANY

	EPX_EXTERNAL_HELPER
		export
			{NONE} all
		end

	CAPI_ERRNO
		export
			{NONE} all
		end

	CAPI_STRING
		export
			{NONE} all
		end


feature -- Commands

	clear is
			-- reset error
		do
			posix_clear_errno
		ensure
			all_clear: is_ok
		end

	set_value (a_value: INTEGER) is
		do
			posix_set_errno (a_value)
		ensure
			value_set: value = a_value
		end


feature -- State

	is_not_ok: BOOLEAN is
			-- Are there any errors?
			-- Only valid if errno has been set.
		do
			Result := value /= 0
		end

	is_ok: BOOLEAN is
			-- Is everything just fine?
			-- Only valid if errno has been set.
		do
			Result := value = 0
		end


feature -- Access

	value: INTEGER is
			-- The error number itself
		do
			Result := posix_errno
		end

	message: STRING is
			-- Current error message
		do
			Result := error_message (value)
		ensure
			message_not_void: Result /= Void
		end

	strerror: STRING is
		obsolete "Use message instead."
		do
			Result := message
		end


feature -- the first, whatever that means,  error value

	clear_first is
			-- Reset notation of first error.
		do
			first_value := 0
		ensure
			no_first_error: first_value = 0
		end

	first_value: INTEGER
			-- Error number of first error that has occurred after a
			-- `clear_first'

	first_message: STRING is
			-- Current error message.
		do
			Result := error_message (first_value)
		ensure
			first_message_not_void: Result /= Void
		end

	set_first is
			-- make `first_value' equal to the current error value
		do
			first_value := value
		ensure
			first_value_set: first_value = value
		end


feature {NONE} -- Implementation

	error_message (an_errno: INTEGER): STRING is
			-- The error message given an error number.
			-- Returns a slightly more helpful error message in case
			-- `an_errno' is unknown.
		local
			p: POINTER
		do
			p := posix_strerror (an_errno)
			Result := sh.pointer_to_string (p)
			if Result.is_equal (once_unknown_error) then
				create Result.make_from_string (once_unknown_error_first)
				Result.append_string (an_errno.out)
				Result.append_string (once_unknown_error_last)
			end
		ensure
			error_message_not_void: Result /= Void
		end


feature {NONE} -- Once strings

	once_unknown_error: STRING is "Unknown error%N"

	once_unknown_error_first: STRING is "Unknown error (value is "
	once_unknown_error_last: STRING is ")%N"

end
